Package: hdxmsqc
Type: Package
Title: An R package for quality Control for hydrogen deuterium exchange
        mass spectrometry experiments
Version: 1.7.0
Authors@R: c(person(family = "Crook", given = "Oliver M.",
		                email = "oliver.crook@stats.ox.ac.uk",
		                comment = c(ORCID = "0000-0001-5669-8506"),
		                role = c("aut", "cre")))
Description: The hdxmsqc package enables us to analyse and visualise the quality
 of HDX-MS experiments. Either as a final quality check before downstream
 analysis and publication or as part of a interative procedure to determine
 the quality of the data. The package builds on the QFeatures and Spectra
 packages to integrate with other mass-spectrometry data.
License: file LICENSE
Encoding: UTF-8
LazyData: false
Depends: R(>= 4.3), QFeatures, S4Vectors, Spectra
Imports: dplyr, tidyr, ggplot2, BiocStyle, knitr, methods, grDevices,
        stats, MsCoreUtils
Suggests: RColorBrewer, pheatmap, MASS, patchwork, testthat
VignetteBuilder: knitr
Roxygen: list(markdown=TRUE)
RoxygenNote: 7.2.3
biocViews: QualityControl,DataImport, Proteomics, MassSpectrometry,
        Metabolomics
BugReports: https://github.com/ococrook/hdxmsqc/issues
URL: http://github.com/ococrook/hdxmsqc
Language: en-US
git_url: https://git.bioconductor.org/packages/hdxmsqc
git_branch: devel
git_last_commit: 1520631
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-10-31 22:16:18 UTC; biocbuild
Author: Oliver M. Crook [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5669-8506>)
Maintainer: Oliver M. Crook <oliver.crook@stats.ox.ac.uk>
