#' This is data to be included in my package
#'
#' @name BRD4df
#' @description
#' 
#' A small HDX-MS dataset for BRD4 in apo state and in complex with IBET151. 
#' This data was generated by GSK and is available for re-use under the package
#' license. The data was first published in Crook et al. 2024 Functional
#' Bayesian model for hydrogen deuterium exchange mass spectrometry, JPR. The
#' data is an object of class `QFeatures` and has been downsampled to 10 peptides
#' and 1 condition to provide example data. See `data("BRD4df_full")` for the
#' complete dataset
#' 
#' @docType data
#' @author Oliver Crook \email{ococrook@gmail.com}
#' @keywords data
NULL

#' This is data to be included in my package
#'
#' @name BRD4df_full
#' @description
#' 
#' A complete HDX-MS dataset for BRD4 in apo state and in complex with IBET151.
#' This data was generated by GSK and is available for re-use under the package
#' license. The data was first published in Crook et al. 2024 Functional
#' Bayesian model for hydrogen deuterium exchange mass spectrometry, JPR. The
#' data is an object of class `QFeatures` and the data has been parsed from a
#' HDExaminer file without filtering or processing. The data contains 179 peptides
#' across 36 samples. THis includes 3 replicate experiments, 2 conditions and
#' 6 timepoints.
#' 
#' 
#' @docType data
#' @author Oliver Crook \email{ococrook@gmail.com}
#' @keywords data
NULL